@@ -1,12 +1,14 @@
+# -*- coding: utf-8 -*-
 ##############################################################################
 #
-# Copyright (c) 2010 Vifib SARL and Contributors. All Rights Reserved.
+# Copyright (c) 2010, 2011, 2012 Vifib SARL and Contributors.
+# All Rights Reserved.
 #
 # WARNING: This program as such is intended to be used by professional
 # programmers who take the whole responsibility of assessing all potential
 # consequences resulting from its eventual inadequacies and bugs
 # End users who are looking for a ready-to-use solution with commercial
-# guarantees and support are strongly adviced to contract a Free Software
+# guarantees and support are strongly advised to contract a Free Software
 # Service Company
 #
 # This program is Free Software; you can redistribute it and/or
@@ -24,6 +26,7 @@
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #
 ##############################################################################
+
 import logging
 import hashlib
 import os
@@ -82,12 +85,13 @@ LOCALE_ENVIRONMENT_REMOVE_LIST = [
   'LC_TIME',
 ]
 
-class AlreadyRunning(Exception):
-  pass
-
 
 class SlapPopen(subprocess.Popen):
-  """Almost normal subprocess with gridish features"""
+  """
+  Almost normal subprocess with greedish features and logging.
+  Each line is logged "live", and self.output is a string containing the whole
+  log.
+  """
   def __init__(self, *args, **kwargs):
     kwargs.update(stdin=subprocess.PIPE)
     subprocess.Popen.__init__(self, *args, **kwargs)
@@ -95,6 +99,18 @@ class SlapPopen(subprocess.Popen):
     self.stdin.close()
     self.stdin = None
 
+    logger = logging.getLogger('SlapProcessManager')
+    # XXX-Cedric: this algorithm looks overkill for simple logging.
+    output_lines = []
+    while True:
+      line = self.stdout.readline()
+      if line == '' and self.poll() != None:
+        break
+      output_lines.append(line)
+      if line[-1:] == '\n':
+        line = line[:-1]
+      logger.info(line)
+    self.output = ''.join(output_lines)
 
 def getSoftwareUrlHash(url):
   return md5(url).hexdigest()
@@ -113,8 +129,8 @@ def getCleanEnvironment(home_path='/tmp'):
       removed_env.append(k)
   changed_env['HOME'] = env['HOME'] = home_path
   for k in sorted(changed_env.iterkeys()):
-    logger.debug('Overriden %s = %r' % (k,changed_env[k]))
-  logger.debug('Removed from environement: %s' % ', '.join(sorted(removed_env)))
+    logger.debug('Overriden %s = %r' % (k, changed_env[k]))
+  logger.debug('Removed from environment: %s' % ', '.join(sorted(removed_env)))
   return env
 
 
@@ -123,19 +139,17 @@ def setRunning(pid_file):
   logger = logging.getLogger('Slapgrid')
   if os.path.exists(pid_file):
     # Pid file is present
-    logger.warning('pid file already exists : %s' % (pid_file))
     try:
       pid = int(open(pid_file, 'r').readline())
     except ValueError:
       pid = None
     # XXX This could use psutil library.
     if pid is not None and os.path.exists("/proc/%s" % pid):
-      #XXX: can we trust sys.argv?
-      process_name = os.path.basename(sys.argv[0])
-      if process_name in open('/proc/%s/cmdline' % pid, 'r').readline():
-        # In case process is present, ignore.
-        raise AlreadyRunning('A slapgrid process is running with pid %s' % pid)
-    logger.info('Pid file %r was stale one, overwritten' % pid_file)
+      # In case process is present, ignore.
+      logger.warning('New slapos process started, but another slapos '
+                     'process is aleady running with pid %s, exiting.' % pid)
+      sys.exit(10)
+    logger.info('Existing pid file %r was stale one, overwritten' % pid_file)
   # Start new process
   write_pid(pid_file)
 
@@ -168,6 +182,8 @@ def dropPrivileges(uid, gid):
   Does nothing in case if uid and gid are not 0
   """
   logger = logging.getLogger('dropPrivileges')
+  # XXX-Cedric: remove format / just do a print, otherwise formatting is done
+  # twice
   current_uid, current_gid = os.getuid(), os.getgid()
   if uid == 0 or gid == 0:
     raise OSError('Dropping privileges to uid = %r or ' \
@@ -213,12 +229,12 @@ def dropPrivileges(uid, gid):
     pass
   else:
     raise ValueError('%s it was possible to go back to uid = %r and gid = '
-        '%r which is fatal.' % message_pre, current_uid, current_gid)
-  logger.info('Succesfully dropped privileges to uid=%r gid=%r' % (uid, gid))
+        '%r which is fatal.' % (message_pre, current_uid, current_gid))
+  logger.debug('Succesfully dropped privileges to uid=%r gid=%r' % (uid, gid))
 
 
 def bootstrapBuildout(path, buildout=None,
-    additional_buildout_parametr_list=None, console=False):
+    additional_buildout_parametr_list=None):
   if additional_buildout_parametr_list is None:
     additional_buildout_parametr_list = []
   logger = logging.getLogger('BuildoutManager')
@@ -234,7 +250,7 @@ def bootstrapBuildout(path, buildout=None,
     invocation_list.extend(additional_buildout_parametr_list)
   else:
     try:
-      import zc.buildout
+      __import__('zc.buildout')
     except ImportError:
       logger.warning('Using old style bootstrap of included bootstrap file. '
         'Consider having zc.buildout available in search path.')
@@ -255,20 +271,14 @@ def bootstrapBuildout(path, buildout=None,
     logger.debug('Set umask from %03o to %03o' % (umask, SAFE_UMASK))
     logger.debug('Invoking: %r in directory %r' % (' '.join(invocation_list),
       path))
-    if not console:
-      kw.update(stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    kw.update(stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
     process_handler = SlapPopen(invocation_list,
             preexec_fn=lambda: dropPrivileges(uid, gid),
             cwd=path, **kw)
-    result = process_handler.communicate()[0]
-    if console:
-      result = 'Please consult messages above'
     if process_handler.returncode is None or process_handler.returncode != 0:
-      message = 'Failed to run buildout profile in directory %r:\n%s\n' % (
-          path, result)
-      raise BuildoutFailedError(message)
-    else:
-      logger.debug('Successful run:\n%s' % result)
+      message = 'Failed to run buildout profile in directory %r' % (path)
+      logger.error(message)
+      raise BuildoutFailedError('%s:\n%s\n' % (message, process_handler.output))
   except OSError as error:
     raise BuildoutFailedError(error)
   finally:
@@ -277,7 +287,7 @@ def bootstrapBuildout(path, buildout=None,
 
 
 def launchBuildout(path, buildout_binary,
-                   additional_buildout_parametr_list=None, console=False):
+                   additional_buildout_parametr_list=None):
   """ Launches buildout."""
   logger = logging.getLogger('BuildoutManager')
   if additional_buildout_parametr_list is None:
@@ -303,21 +313,14 @@ def launchBuildout(path, buildout_binary,
     logger.debug('Set umask from %03o to %03o' % (umask, SAFE_UMASK))
     logger.debug('Invoking: %r in directory %r' % (' '.join(invocation_list),
       path))
-    kw = dict()
-    if not console:
-      kw.update(stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    kw = dict(stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
     process_handler = SlapPopen(invocation_list,
             preexec_fn=lambda: dropPrivileges(uid, gid), cwd=path,
             env=getCleanEnvironment(pwd.getpwuid(uid).pw_dir), **kw)
-    result = process_handler.communicate()[0]
-    if console:
-      result = 'Please consult messages above'
     if process_handler.returncode is None or process_handler.returncode != 0:
-      message = 'Failed to run buildout profile in directory %r:\n%s\n' % (
-          path, result)
-      raise BuildoutFailedError(message)
-    else:
-      logger.debug('Successful run:\n%s' % result)
+      message = 'Failed to run buildout profile in directory %r' % (path)
+      logger.error(message)
+      raise BuildoutFailedError('%s:\n%s\n' % (message, process_handler.output))
   except OSError as error:
     raise BuildoutFailedError(error)
   finally:
@@ -329,13 +332,13 @@ def updateFile(file_path, content, mode='0600'):
   """Creates an executable with "content" as content."""
   altered = False
   if not (os.path.isfile(file_path)) or \
-    not(hashlib.md5(open(file_path).read()).digest() ==\
-        hashlib.md5(content).digest()):
-      altered = True
-      file_file = open(file_path, 'w')
-      file_file.write(content)
-      file_file.flush()
-      file_file.close()
+     not(hashlib.md5(open(file_path).read()).digest() ==\
+         hashlib.md5(content).digest()):
+    altered = True
+    file_file = open(file_path, 'w')
+    file_file.write(content)
+    file_file.flush()
+    file_file.close()
   os.chmod(file_path, stat.S_IREAD | stat.S_IWRITE | stat.S_IEXEC)
   if oct(stat.S_IMODE(os.stat(file_path).st_mode)) != mode:
     os.chmod(file_path, int(mode, 8))
@@ -357,4 +360,4 @@ def createPrivateDirectory(path):
   if permission not in ('0700'):
     raise WrongPermissionError('Wrong permissions in %s ' \
                                         ': is %s, should be 0700'
-                                        % path, permission)
+                                        % (path, permission))
