@@ -45,10 +45,12 @@ class SlapMixin(unittest.TestCase):
       self.server_url = 'http://localhost/'
     else:
       self.server_url = self._server_url
+    print 'Testing against SLAP server %r' % self.server_url
+    self.slap = slapos.slap.slap()
+    self.partition_id = 'PARTITION_01'
 
   def tearDown(self):
-    if self._server_url is None:
-      self._unpatchHttplib()
+    self._unpatchHttplib()
 
   def _patchHttplib(self):
     """Overrides httplib"""
@@ -81,27 +83,29 @@ class TestSlap(SlapMixin):
 
   def test_slap_initialisation(self):
     """
-    Asserts that slap initialisation works properly in case of 
+    Asserts that slap initialisation works properly in case of
     passing correct url
     """
     slap_instance = slapos.slap.slap()
     slap_instance.initializeConnection(self.server_url)
-    self.assertTrue(slapos.slap.SlapDocument._connection_helper.host in self.server_url)
-    self.assertTrue(slapos.slap.SlapDocument._connection_helper.path in self.server_url)
+    self.assertTrue(
+        slap_instance._connection_helper.host in self.server_url)
+    self.assertTrue(
+        slap_instance._connection_helper.path in self.server_url)
 
   def test_slap_initialisation_wrong_url(self):
     """
-    Asserts that slap initialisation raises exception when passed url 
+    Asserts that slap initialisation raises exception when passed url
     is not correct
     """
     server_url = 'https://user:pass@server/path/path?parameter=notAcceptable'
     slap_instance = slapos.slap.slap()
-    self.assertRaises(AttributeError, 
+    self.assertRaises(AttributeError,
                       slap_instance.initializeConnection, server_url)
 
   def test_registerComputer_with_new_guid(self):
     """
-    Asserts that calling slap.registerComputer with new guid returns 
+    Asserts that calling slap.registerComputer with new guid returns
     Computer object
     """
     computer_guid = self._getTestComputerId()
@@ -112,7 +116,7 @@ class TestSlap(SlapMixin):
 
   def test_registerComputer_with_existing_guid(self):
     """
-    Asserts that calling slap.registerComputer with already used guid 
+    Asserts that calling slap.registerComputer with already used guid
     returns Computer object
     """
     computer_guid = self._getTestComputerId()
@@ -128,7 +132,7 @@ class TestSlap(SlapMixin):
   # SoftwareRelease is currently used as suboject of Slap transmission object
   def test_registerSoftwareRelease_with_new_uri(self):
     """
-    Asserts that calling slap.registerSoftwareRelease with new guid 
+    Asserts that calling slap.registerSoftwareRelease with new guid
     returns SoftwareRelease object
     """
     software_release_uri = 'http://server/' + self._getTestComputerId()
@@ -140,7 +144,7 @@ class TestSlap(SlapMixin):
 
   def test_registerSoftwareRelease_with_existing_uri(self):
     """
-    Asserts that calling slap.registerSoftwareRelease with already 
+    Asserts that calling slap.registerSoftwareRelease with already
     used guid returns SoftwareRelease object
     """
     software_release_uri = 'http://server/' + self._getTestComputerId()
@@ -156,43 +160,130 @@ class TestSlap(SlapMixin):
 
   def test_registerComputerPartition_new_partition_id_known_computer_guid(self):
     """
-    Asserts that calling slap.registerComputerPartition on known computer 
+    Asserts that calling slap.registerComputerPartition on known computer
     returns ComputerPartition object
     """
-    self.computer_guid = self._getTestComputerId()
-    self.slap = slapos.slap.slap()
+    computer_guid = self._getTestComputerId()
+    partition_id = self.partition_id
     self.slap.initializeConnection(self.server_url)
-    self.partition_id = 'PARTITION_01'
-    self.slap.registerComputer(self.computer_guid)
+    self.slap.registerComputer(computer_guid)
 
-    partition = self.slap.registerComputerPartition(self.computer_guid, 
-                                                    self.partition_id)
+    def server_response(self, path, method, body, header):
+      parsed_url = urlparse.urlparse(path.lstrip('/'))
+      parsed_qs = urlparse.parse_qs(parsed_url.query)
+      if parsed_url.path == 'registerComputerPartition' and \
+         parsed_qs['computer_reference'][0] == computer_guid and \
+         parsed_qs['computer_partition_reference'][0] == partition_id:
+        partition = slapos.slap.ComputerPartition(
+            computer_guid, partition_id)
+        return (200, {}, xml_marshaller.xml_marshaller.dumps(partition))
+      else:
+        return (404, {}, '')
+    httplib.HTTPConnection._callback = server_response
+
+    partition = self.slap.registerComputerPartition(computer_guid, partition_id)
     self.assertTrue(isinstance(partition, slapos.slap.ComputerPartition))
 
   def test_registerComputerPartition_existing_partition_id_known_computer_guid(self):
     """
-    Asserts that calling slap.registerComputerPartition on known computer 
+    Asserts that calling slap.registerComputerPartition on known computer
     returns ComputerPartition object
     """
     self.test_registerComputerPartition_new_partition_id_known_computer_guid()
-    partition = self.slap.registerComputerPartition(self.computer_guid,
+    partition = self.slap.registerComputerPartition(self._getTestComputerId(),
         self.partition_id)
     self.assertTrue(isinstance(partition, slapos.slap.ComputerPartition))
 
   def test_registerComputerPartition_unknown_computer_guid(self):
     """
-    Asserts that calling slap.registerComputerPartition on unknown 
-    computer raises (not defined yet) exception
+    Asserts that calling slap.registerComputerPartition on unknown
+    computer raises NotFoundError exception
     """
     computer_guid = self._getTestComputerId()
-    self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
-
     partition_id = 'PARTITION_01'
 
-    self.assertRaises(UndefinedYetException,
+    def server_response(self, path, method, body, header):
+      parsed_url = urlparse.urlparse(path.lstrip('/'))
+      parsed_qs = urlparse.parse_qs(parsed_url.query)
+      if parsed_url.path == 'registerComputerPartition' and \
+         parsed_qs['computer_reference'][0] == computer_guid and \
+         parsed_qs['computer_partition_reference'][0] == partition_id:
+        slapos.slap.ComputerPartition(computer_guid, partition_id)
+        return (404, {}, '')
+      else:
+        return (0, {}, '')
+    httplib.HTTPConnection._callback = server_response
+
+    self.assertRaises(slapos.slap.NotFoundError,
         self.slap.registerComputerPartition, computer_guid, partition_id)
 
+  def test_getFullComputerInformation_empty_computer_guid(self):
+    """
+    Asserts that calling getFullComputerInformation with empty computer_id
+    raises early, before calling master.
+    """
+    self.slap.initializeConnection(self.server_url)
+
+    def server_response(self_httpconnection, path, method, body, header):
+      # Shouldn't even be called
+      self.assertFalse(True)
+    httplib.HTTPConnection._callback = server_response
+
+    self.assertRaises(
+        slapos.slap.NotFoundError,
+        self.slap._connection_helper.getFullComputerInformation, None)
+
+  def test_registerComputerPartition_empty_computer_guid(self):
+    """
+    Asserts that calling registerComputerPartition with empty computer_id
+    raises early, before calling master.
+    """
+    self.slap.initializeConnection(self.server_url)
+
+    def server_response(self_httpconnection, path, method, body, header):
+      # Shouldn't even be called
+      self.assertFalse(True)
+    httplib.HTTPConnection._callback = server_response
+
+    self.assertRaises(
+        slapos.slap.NotFoundError,
+        self.slap.registerComputerPartition, None, 'PARTITION_01')
+
+  def test_registerComputerPartition_empty_computer_partition_id(self):
+    """
+    Asserts that calling registerComputerPartition with empty
+    computer_partition_id raises early, before calling master.
+    """
+    self.slap.initializeConnection(self.server_url)
+
+    def server_response(self_httpconnection, path, method, body, header):
+      # Shouldn't even be called
+      self.assertFalse(True)
+    httplib.HTTPConnection._callback = server_response
+
+    self.assertRaises(
+        slapos.slap.NotFoundError,
+        self.slap.registerComputerPartition,
+        self._getTestComputerId(), None)
+
+  def test_registerComputerPartition_empty_computer_guid_empty_computer_partition_id(self):
+    """
+    Asserts that calling registerComputerPartition with empty
+    computer_partition_id raises early, before calling master.
+    """
+    self.slap.initializeConnection(self.server_url)
+
+    def server_response(self_httpconnection, path, method, body, header):
+      # Shouldn't even be called
+      self.assertFalse(True)
+    httplib.HTTPConnection._callback = server_response
+
+    self.assertRaises(
+        slapos.slap.NotFoundError,
+        self.slap.registerComputerPartition,
+        None, None)
+
 class TestComputer(SlapMixin):
   """
   Tests slapos.slap.slap.Computer class functionality
@@ -203,15 +294,69 @@ class TestComputer(SlapMixin):
     Asserts that calling Computer.getComputerPartitionList without Computer
     Partitions returns empty list
     """
-    self.computer_guid = self._getTestComputerId()
-    self.slap = slapos.slap.slap()
+    computer_guid = self._getTestComputerId()
+    slap = self.slap
+    slap.initializeConnection(self.server_url)
+
+    def server_response(self, path, method, body, header):
+      parsed_url = urlparse.urlparse(path.lstrip('/'))
+      parsed_qs = urlparse.parse_qs(parsed_url.query)
+      if parsed_url.path == 'registerComputerPartition' and \
+         'computer_reference' in parsed_qs and \
+         'computer_partition_reference' in parsed_qs:
+        slap_partition = slapos.slap.ComputerPartition(
+          parsed_qs['computer_reference'][0],
+          parsed_qs['computer_partition_reference'][0])
+        return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_partition))
+      elif parsed_url.path == 'getFullComputerInformation' and \
+         'computer_id' in parsed_qs:
+        slap_computer = slapos.slap.Computer(parsed_qs['computer_id'][0])
+        slap_computer._software_release_list = []
+        slap_computer._computer_partition_list = []
+        return (200, {}, xml_marshaller.xml_marshaller.dumps(slap_computer))
+      elif parsed_url.path == 'requestComputerPartition':
+        return (408, {}, '')
+      else:
+        return (404, {}, '')
+    httplib.HTTPConnection._callback = server_response
+
+    computer = self.slap.registerComputer(computer_guid)
+    self.assertEqual(computer.getComputerPartitionList(), [])
+
+  def _test_computer_empty_computer_guid(self, computer_method):
+    """
+    Helper method checking if calling Computer method with empty id raises
+    early.
+    """
     self.slap.initializeConnection(self.server_url)
-    self.computer = self.slap.registerComputer(self.computer_guid)
-    self.assertEqual(self.computer.getComputerPartitionList(), [])
+
+    def server_response(self_httpconnection, path, method, body, header):
+      # Shouldn't even be called
+      self.assertFalse(True)
+    httplib.HTTPConnection._callback = server_response
+
+    computer = self.slap.registerComputer(None)
+    self.assertRaises(
+        slapos.slap.NotFoundError,
+        getattr(computer, computer_method))
+
+  def test_computer_getComputerPartitionList_empty_computer_guid(self):
+    """
+    Asserts that calling getComputerPartitionList with empty
+    computer_guid raises early, before calling master.
+    """
+    self._test_computer_empty_computer_guid('getComputerPartitionList')
+
+  def test_computer_getSoftwareReleaseList_empty_computer_guid(self):
+    """
+    Asserts that calling getSoftwareReleaseList with empty
+    computer_guid raises early, before calling master.
+    """
+    self._test_computer_empty_computer_guid('getSoftwareReleaseList')
 
   def test_computer_getComputerPartitionList_only_partition(self):
     """
-    Asserts that calling Computer.getComputerPartitionList with only 
+    Asserts that calling Computer.getComputerPartitionList with only
     Computer Partitions returns empty list
     """
     self.computer_guid = self._getTestComputerId()
@@ -223,11 +368,14 @@ class TestComputer(SlapMixin):
         partition_id)
     self.assertEqual(self.computer.getComputerPartitionList(), [])
 
+  @unittest.skip("Not implemented")
   def test_computer_reportUsage_non_valid_xml_raises(self):
     """
-    Asserts that calling Computer.reportUsage with non DTD 
+    Asserts that calling Computer.reportUsage with non DTD
     (not defined yet) XML raises (not defined yet) exception
     """
+
+
     self.computer_guid = self._getTestComputerId()
     self.slap = slapos.slap.slap()
     self.slap.initializeConnection(self.server_url)
@@ -238,9 +386,10 @@ class TestComputer(SlapMixin):
     self.assertRaises(UndefinedYetException, self.computer.reportUsage,
         non_dtd_xml)
 
+  @unittest.skip("Not implemented")
   def test_computer_reportUsage_valid_xml_invalid_partition_raises(self):
     """
-    Asserts that calling Computer.reportUsage with DTD (not defined 
+    Asserts that calling Computer.reportUsage with DTD (not defined
     yet) XML which refers to invalid partition raises (not defined yet)
     exception
     """
@@ -268,7 +417,6 @@ class TestComputerPartition(SlapMixin):
 
   def test_request_sends_request(self):
     partition_id = 'PARTITION_01'
-    request_called_amount = 0
     def server_response(self, path, method, body, header):
       parsed_url = urlparse.urlparse(path.lstrip('/'))
       parsed_qs = urlparse.parse_qs(parsed_url.query)
@@ -379,7 +527,8 @@ class TestComputerPartition(SlapMixin):
     self.assertTrue(isinstance(requested_partition,
         slapos.slap.ComputerPartition))
     # as request method does not raise, accessing data raises
-    self.assertRaises(slapos.slap.ResourceNotReady, requested_partition.getId)
+    self.assertRaises(slapos.slap.ResourceNotReady,
+                      requested_partition.getId)
 
   def test_request_fullfilled_work(self):
     partition_id = 'PARTITION_01'
@@ -429,42 +578,56 @@ class TestComputerPartition(SlapMixin):
 
   def _test_new_computer_partition_state(self, state):
     """
-    Helper method to automate assertions of failing states on new Computer 
+    Helper method to automate assertions of failing states on new Computer
     Partition
     """
-    self.computer_guid = self._getTestComputerId()
+    computer_guid = self._getTestComputerId()
     partition_id = 'PARTITION_01'
-    self.slap = slapos.slap.slap()
-    self.slap.initializeConnection(self.server_url)
+    slap = self.slap
+    slap.initializeConnection(self.server_url)
+
+    def server_response(self, path, method, body, header):
+      parsed_url = urlparse.urlparse(path.lstrip('/'))
+      parsed_qs = urlparse.parse_qs(parsed_url.query)
+      if parsed_url.path == 'registerComputerPartition' and \
+         parsed_qs['computer_reference'][0] == computer_guid and \
+         parsed_qs['computer_partition_reference'][0] == partition_id:
+        partition = slapos.slap.ComputerPartition(
+            computer_guid, partition_id)
+        return (200, {}, xml_marshaller.xml_marshaller.dumps(partition))
+      else:
+        return (404, {}, '')
+    httplib.HTTPConnection._callback = server_response
+
     computer_partition = self.slap.registerComputerPartition(
-        self.computer_guid, partition_id)
+        computer_guid, partition_id)
     method = getattr(computer_partition, state)
-    self.assertRaises(UndefinedYetException, method)
+    self.assertRaises(slapos.slap.NotFoundError, method)
 
   def test_available_new_ComputerPartition_raises(self):
     """
-    Asserts that calling ComputerPartition.available on new partition 
+    Asserts that calling ComputerPartition.available on new partition
     raises (not defined yet) exception
     """
     self._test_new_computer_partition_state('available')
 
   def test_building_new_ComputerPartition_raises(self):
     """
-    Asserts that calling ComputerPartition.building on new partition raises 
+    Asserts that calling ComputerPartition.building on new partition raises
     (not defined yet) exception
     """
     self._test_new_computer_partition_state('building')
 
   def test_started_new_ComputerPartition_raises(self):
     """
-    Asserts that calling ComputerPartition.started on new partition raises 
+    Asserts that calling ComputerPartition.started on new partition raises
     (not defined yet) exception
     """
     self._test_new_computer_partition_state('started')
 
   def test_stopped_new_ComputerPartition_raises(self):
     """
-    Asserts that calling ComputerPartition.stopped on new partition raises 
+    Asserts that calling ComputerPartition.stopped on new partition raises
     (not defined yet) exception
     """
     self._test_new_computer_partition_state('stopped')
@@ -473,12 +636,35 @@ class TestComputerPartition(SlapMixin):
     """
     Asserts that calling ComputerPartition.error on new partition works
     """
-    self.computer_guid = self._getTestComputerId()
+    computer_guid = self._getTestComputerId()
     partition_id = 'PARTITION_01'
-    self.slap = slapos.slap.slap()
-    self.slap.initializeConnection(self.server_url)
-    computer_partition = self.slap.registerComputerPartition(
-        self.computer_guid, partition_id)
+    slap = self.slap
+    slap.initializeConnection(self.server_url)
+
+    def server_response(self, path, method, body, header):
+      parsed_url = urlparse.urlparse(path.lstrip('/'))
+      parsed_qs = urlparse.parse_qs(parsed_url.query)
+      if parsed_url.path == 'registerComputerPartition' and \
+         parsed_qs['computer_reference'][0] == computer_guid and \
+         parsed_qs['computer_partition_reference'][0] == partition_id:
+        partition = slapos.slap.ComputerPartition(
+            computer_guid, partition_id)
+        return (200, {}, xml_marshaller.xml_marshaller.dumps(partition))
+      elif parsed_url.path == 'softwareInstanceError':
+        parsed_qs_body = urlparse.parse_qs(body)
+        # XXX: why do we have computer_id and not computer_reference?
+        # XXX: why do we have computer_partition_id and not
+        # computer_partition_reference?
+        if parsed_qs_body['computer_id'][0] == computer_guid and \
+           parsed_qs_body['computer_partition_id'][0] == partition_id and \
+           parsed_qs_body['error_log'][0] == 'some error':
+          return (200, {}, '')
+
+      return (404, {}, '')
+    httplib.HTTPConnection._callback = server_response
+
+    computer_partition = slap.registerComputerPartition(
+        computer_guid, partition_id)
     # XXX: Interface does not define return value
     computer_partition.error('some error')
 
@@ -489,7 +675,7 @@ class TestSoftwareRelease(SlapMixin):
 
   def _test_new_software_release_state(self, state):
     """
-    Helper method to automate assertions of failing states on new Software 
+    Helper method to automate assertions of failing states on new Software
     Release
     """
     self.software_release_uri = 'http://server/' + self._getTestComputerId()
@@ -498,32 +684,44 @@ class TestSoftwareRelease(SlapMixin):
     software_release = self.slap.registerSoftwareRelease(
         self.software_release_uri)
     method = getattr(software_release, state)
-    self.assertRaises(UndefinedYetException, method)
+    self.assertRaises(NameError, method)
 
   def test_available_new_SoftwareRelease_raises(self):
     """
-    Asserts that calling SoftwareRelease.available on new software release 
-    raises (not defined yet) exception
+    Asserts that calling SoftwareRelease.available on new software release
+    raises NameError exception
     """
     self._test_new_software_release_state('available')
 
   def test_building_new_SoftwareRelease_raises(self):
     """
-    Asserts that calling SoftwareRelease.building on new software release 
-    raises (not defined yet) exception
+    Asserts that calling SoftwareRelease.building on new software release
+    raises NameError exception
     """
     self._test_new_software_release_state('building')
 
   def test_error_new_SoftwareRelease_works(self):
     """
-    Asserts that calling SoftwareRelease.error on new software release works
+    Asserts that calling SoftwareRelease.error on software release works
     """
-    self.software_release_uri = 'http://server/' + self._getTestComputerId()
-    self.slap = slapos.slap.slap()
-    self.slap.initializeConnection(self.server_url)
-    software_release = self.slap.registerSoftwareRelease(
-        self.software_release_uri)
-    # XXX: Interface does not define return value
+    computer_guid = self._getTestComputerId()
+    software_release_uri = 'http://server/' + self._getTestComputerId()
+    slap = self.slap
+    slap.initializeConnection(self.server_url)
+
+    def server_response(self, path, method, body, header):
+      parsed_url = urlparse.urlparse(path.lstrip('/'))
+      parsed_qs = urlparse.parse_qs(body)
+      if parsed_url.path == 'softwareReleaseError' and \
+         parsed_qs['computer_id'][0] == computer_guid and \
+         parsed_qs['url'][0] == software_release_uri and \
+         parsed_qs['error_log'][0] == 'some error':
+        return (200, {}, '')
+      return (404, {}, '')
+    httplib.HTTPConnection._callback = server_response
+
+    software_release = self.slap.registerSoftwareRelease(software_release_uri)
+    software_release._computer_guid = computer_guid
     software_release.error('some error')
 
 class TestOpenOrder(SlapMixin):
@@ -574,8 +772,6 @@ class TestOpenOrder(SlapMixin):
     computer_guid = self._getTestComputerId()
     requested_partition_id = 'PARTITION_01'
     def server_response(self, path, method, body, header):
-      parsed_url = urlparse.urlparse(path.lstrip('/'))
-      parsed_qs = urlparse.parse_qs(parsed_url.query)
       from slapos.slap.slap import SoftwareInstance
       slap_partition = SoftwareInstance(
         slap_computer_id=computer_guid,
@@ -589,7 +785,6 @@ class TestOpenOrder(SlapMixin):
     self.assertTrue(requested_partition_id, computer_partition.getId())
 
 if __name__ == '__main__':
-  print 'Testing against SLAP server %r' % SERVER_URL
   print 'You can point to any SLAP server by setting TEST_SLAP_SERVER_URL '\
       'environment variable'
   unittest.main()
